;Codigo del tone en ensamblador de Clipper
	PAGE	,132
	NAME    EXAMPLES
;************
;
;	examplea.asm
;
;	This file contains demonstration programs using the 
;	CLIPPER assembly interface.
;
;	Function source.....
;
;		TONE()
;
	INCLUDE	EXTENDA.INC


;***************
;
;	define segment names
;
	CODESEG	EXAMPLEA
	DATASEG


;***************
;
;	declaration of public functions
;
	CLpublic <TONE>


;***************
;
;	Static data
;
	CLstatic <long nbuff 0>

	; storage for full pathname and pointer to address it
	CLstatic <byte ASCIIZ <<64 DUP(0)>>, cptr PATH ASCIIZ>


;***************
;
;	Equates
;

$define FALSE			0000H
$define TRUE			0001H

	; for TONE()
$define TIMER_COMMAND		00B6H		; value for timer setup
$define TIMER_COMMAND_REGISTER	0043H		; port address
$define TIMER_LATCH_REGISTER	0042H		; port address
$define PORT_PB			0061H		; port address
$define SPEAKER_ON		00000011B 	; bit mask
$define CLOCK_RATE_HIGH		0012H		; 1.19318 MHz as 32 bit value
$define CLOCK_RATE_LOW		34DCH

	
;******
;
;	declare dummy segment to access BIOS data area
;	no code is generated by this declaration
;
BIOS_DATA	SEGMENT AT 40H

		ORG	6CH
TIMER_LOW	LABEL	WORD			; time of day is stored here

BIOS_DATA	ENDS


;***************
;
;	begin local assembler routines
;
	WORKFUNCS


;******
;
;	delay processing a specified number of timer tics
;	the count is contained in the DX register
;	each tic is 1/18 second
;
TIME_DELAY	PROC	NEAR

		PUSH	ES			; preserve


	; ES = segment of BIOS data area
		MOV	AX, BIOS_DATA		; load segment value
		MOV	ES, AX

		ASSUME	ES:BIOS_DATA		; (tell MASM what's in ES)


	; delay at least one tick
		MOV	BX, TIMER_LOW		; fetch current time
T1:
		CMP	BX, TIMER_LOW		; loop 'til TIMER_LOW changes
		JE	T1


	; delay loop
T2:
		MOV	AX, TIMER_LOW		; fetch current time
		SUB	AX, BX			; AX = number of ticks so far
		CMP	AX, DX			; compare with requested delay
		JL	T2			; keep looping if AX < DX


		POP	ES			; restore
		ASSUME	ES:NOTHING		; (tell MASM nothing in ES)

		RET				; near return

TIME_DELAY	ENDP


;***************
;
;	end of local assembler routines
;
	ENDWORK


;******
;
;	TONE()
;	Dennis L. Dias
;	12/09/87
;
;	Produce a tone of the specified frequency and duration.  If the
;	frequency is less than 20 Hz wait the specified duration without
;	producing a tone.
;
;	CAUTION:  this routine works only on IBM-PC compatible hardware!
;
;	TONE(frequency, duration)
;
;		frequency:	numeric -- cycles per second
;		duration:	numeric -- 18ths of a second
;
	CLfunc void TONE <int frequency, int duration>

	CLcode
		XOR	DX, DX			; delay one timer tic
		CALL	TIME_DELAY

	; 1.19318 MHz / frequency
		MOV	BX, frequency
		CMP	BX, 20
		JB	SILENCE

	; select timer channel 2; read/write LSB, MSB; mode 3; binary
		MOV	AL, TIMER_COMMAND
		OUT	TIMER_COMMAND_REGISTER, AL

		MOV	DX, CLOCK_RATE_HIGH
		MOV	AX, CLOCK_RATE_LOW

		DIV	BX			; AX = correct Hz divisor

	; output LSB, MSB (this sets the frequency)
		OUT	TIMER_LATCH_REGISTER, AL
		MOV	AL, AH
		OUT	TIMER_LATCH_REGISTER, AL

	; save current setting of port pb
		IN	AL, PORT_PB
		PUSH	AX

	; turn on speaker data and timer gate bits
		OR	AL, SPEAKER_ON
		OUT	PORT_PB, AL

	; speaker is on..wait the requested duration
		MOV	DX, duration
		CALL	TIME_DELAY

	; shut that thing off!!
		POP	AX
		OUT	PORT_PB, AL

		JMP	SHORT TONE_RET

SILENCE:
	; just delay
		MOV	DX, duration
		CALL	TIME_DELAY

TONE_RET:

	; no return value
	CLret


; eof examplea.asm